@@ -17,7 +17,7 @@ use Encode qw(encode decode);
 use Carp qw(carp croak);
 
 use vars qw'$VERSION %link_spec';
-$VERSION = '0.36';
+$VERSION = '0.37';
 
 =head1 NAME
 
@@ -753,6 +753,7 @@ with L<WWW::Mechanize>.
 sub get {
     my ($self,$url, %options) = @_;
     my $b = $self->tab->{linkedBrowser};
+    $self->clear_current_form;
 
     if (my $target = delete $options{":content_file"}) {
         $self->save_url($url => $target, %options);
@@ -795,7 +796,7 @@ sub _addEventListener {
     $events = [$events]
         unless ref $events;
 
-# This registers multiple events for a one-shot event
+    # This registers multiple events for a one-shot event
     my $make_semaphore = $self->repl->declare(<<'JS');
 function(browser,events) {
     var lock = {};
@@ -1633,7 +1634,7 @@ use vars '%xpath_quote';
 # Return the default limiter if no other limiting option is set:
 sub _default_limiter {
     my ($default, $options) = @_;
-    if (! grep { exists $options->{ $_ } } qw(single one maybe all)) {
+    if (! grep { exists $options->{ $_ } } qw(single one maybe all any)) {
         $options->{ $default } = 1;
     };
     return ()
@@ -1827,7 +1828,6 @@ sub find_all_links_dom {
     return \@matches;
 };
 
-
 =head2 C<< $mech->follow_link $link >>
 
 =head2 C<< $mech->follow_link %options >>
@@ -1856,6 +1856,185 @@ sub follow_link {
     }
 }
 
+=head2 C<< $mech->xpath $query, %options >>
+
+    my $link = $mech->xpath('//a[id="clickme"]', one => 1);
+    # croaks if there is no link or more than one link found
+
+    my @para = $mech->xpath('//p');
+    # Collects all paragraphs
+
+Runs an XPath query in Firefox against the current document.
+
+The options allow the following keys:
+
+=over 4
+
+=item *
+
+C<< document >> - document in which the query is to be executed. Use this to
+search a node within a specific subframe of C<< $mech->document >>.
+
+=item *
+
+C<< frames >> - if true, search all documents in all frames and iframes.
+This may or may not conflict with C<node>. This will default to the
+C<frames> setting of the WWW::Mechanize::Firefox object.
+
+=item *
+
+C<< node >> - node relative to which the query is to be executed
+
+=item *
+
+C<< single >> - If true, ensure that only one element is found. Otherwise croak
+or carp, depending on the C<autodie> parameter.
+
+=item *
+
+C<< one >> - If true, ensure that at least one element is found. Otherwise croak
+or carp, depending on the C<autodie> parameter.
+
+=item *
+
+C<< maybe >> - If true, ensure that at most one element is found. Otherwise
+croak or carp, depending on the C<autodie> parameter.
+
+=item *
+
+C<< all >> - If true, return all elements found. This is the default.
+You can use this option if you want to use C<< ->xpath >> in scalar context
+to count the number of matched elements, as it will otherwise emit a warning
+for each usage in scalar context without any of the above restricting options.
+
+=item *
+
+C<< any >> - no error is raised, no matter if an item is found or not.
+
+=back
+
+Returns the matched nodes.
+
+You can pass in a list of queries as an array reference for the first parameter.
+
+This is a method that is not implemented in WWW::Mechanize.
+
+In the long run, this should go into a general plugin for
+L<WWW::Mechanize>.
+
+=cut
+
+sub xpath {
+    my ($self,$query,%options) = @_;
+    if ('ARRAY' ne (ref $query||'')) {
+        $query = [$query];
+    };
+    
+    if ($options{ node }) {
+        $options{ document } ||= $options{ node }->{ownerDocument};
+        #warn "Have node, searching below node";
+    } else {
+        $options{ document } ||= $self->document;
+        #$options{ node } = $options{ document };
+    };
+    
+    $options{ user_info } ||= join " or ", map {qq{'$_'}} @$query;
+    my $single = delete $options{ single };
+    my $first  = delete $options{ one };
+    my $maybe  = delete $options{ maybe };
+    my $any    = delete $options{ any };
+    
+    # Construct some helper variables
+    my $zero_allowed = not ($single or $first);
+    my $two_allowed  = not( $single or $maybe);
+    my $return_first = ($single or $first or $maybe or $any);
+    #warn "Zero: $zero_allowed";
+    #warn "Two : $two_allowed";
+    #warn "Ret : $return_first";
+    
+    # Sanity check for the common error of
+    # my $item = $mech->xpath("//foo");
+    if (! exists $options{ all } and not ($return_first)) {
+        $self->signal_condition(join "\n",
+            "You asked for many elements but seem to only want a single item.",
+            "Did you forget to pass the 'single' option with a true value?",
+            "Pass 'all => 1' to suppress this message and receive the count of items.",
+        ) if defined wantarray and !wantarray;
+    };
+    
+    if (not exists $options{ frames }) {
+        $options{frames} = $self->{frames};
+    };
+    
+    my @res;
+    
+    DOCUMENTS: {            
+        my @documents = $options{ document };
+        #warn "Invalid root document" unless $options{ document };
+        
+        # recursively join the results of sub(i)frames if wanted
+        # This should maybe go into the loop to expand every frame as we descend
+        # into the available subframes
+
+        while (@documents) {
+            my $doc = shift @documents;
+            #warn "Invalid document" unless $doc;
+
+            my $n = $options{ node } || $doc;
+            #warn "$nesting>Searching @$query in $doc->{title}";
+            push @res, map { $doc->__xpath($_, $n) } @$query;
+            
+            # A small optimization to return if we already have enough elements
+            # We can't do this on $return_first as there might be more elements
+            last DOCUMENTS if @res and $first;        
+            
+            if ($options{ frames } and not $options{ node }) {
+                #warn "$nesting>Expanding below " . $doc->{title};
+                #local $nesting .= "--";
+                my @d = $self->expand_frames( $options{ frames }, $doc );
+                #warn "Found $_->{title}" for @d;
+                push @documents, @d;
+            };
+        };
+    };
+    
+    if (! $zero_allowed and @res == 0) {
+        $self->signal_condition( "No elements found for $options{ user_info }" );
+    };
+    
+    if (! $two_allowed and @res > 1) {
+        $self->highlight_node(@res);
+        $self->signal_condition( (scalar @res) . " elements found for $options{ user_info }" );
+    };
+    
+    return $return_first ? $res[0] : @res;
+};
+
+=head2 C<< $mech->selector $css_selector, %options >>
+
+  my @text = $mech->selector('p.content');
+
+Returns all nodes matching the given CSS selector. If
+$css_selector is an array reference, it returns
+all nodes matched by any of the CSS selectors in the array.
+
+This takes the same options that C<< ->xpath >> does.
+
+In the long run, this should go into a general plugin for
+L<WWW::Mechanize>.
+
+=cut
+
+sub selector {
+    my ($self,$query,%options) = @_;
+    $options{ user_info } ||= "CSS selector '$query'";
+    if ('ARRAY' ne (ref $query || '')) {
+        $query = [$query];
+    };
+    my @q = map { selector_to_xpath($_); } @$query;
+    $self->xpath(\@q, %options);
+};
+
 =head2 C<< $mech->click $name [,$x ,$y] >>
 
   $mech->click( 'go' );
@@ -1969,11 +2148,26 @@ This method is incompatible with L<WWW::Mechanize>.
 It returns the DOM C<< <form> >> object and not
 a L<HTML::Form> instance.
 
+Note that WWW::Mechanize::Firefox has little way to know
+that the current form is not displayed in the browser
+anymore, so it often holds on to the last value. If
+you want to make sure that a fresh or no form is used,
+remove it:
+
+    $mech->clear_current_form;
+    
+The current form will be reset by WWW::Mechanize::Firefox
+on calls to C<< ->get() >> and C<< ->get_local() >>,
+and on calls to C<< ->submit() >> and C<< ->submit_with_fields >>.
+
 =cut
 
 sub current_form {
     $_[0]->{current_form}
 };
+sub clear_current_form {
+    undef $_[0]->{current_form};
+};
 
 =head2 C<< $mech->form_name $name [, %options] >>
 
@@ -1989,7 +2183,7 @@ sub form_name {
     $name = quote_xpath $name;
     _default_limiter( single => \%options );
     $self->{current_form} = $self->selector("form[name='$name']",
-        user_info => "form id '$name'",
+        user_info => "form name '$name'",
         %options
     );
 };
@@ -2410,7 +2604,7 @@ sub tick {
         if ($set) {
             $target->{checked}= 'checked';
         } else {
-            $target->{checked}= undef;
+            $target->{checked} = 0;
         };
     };
 };
@@ -2445,6 +2639,7 @@ sub submit {
     $dom_form ||= $self->current_form;
     if ($dom_form) {
         $dom_form->submit();
+        $self->clear_current_form;
         1;
     } else {
         croak "I don't know which form to submit, sorry.";
@@ -2626,6 +2821,10 @@ C<xpath> - the XPath query
 
 C<selector> - the CSS selector
 
+=item *
+
+C<dom> - a DOM node
+
 =back
 
 The remaining options are passed through to either the
@@ -2679,7 +2878,7 @@ sub is_visible {
         return false
     }
 JS
-    $_is_visible->($options{dom});
+    !!$_is_visible->($options{dom});
 };
 
 =head2 C<< $mech->wait_until_invisible $element >>
@@ -2777,180 +2976,6 @@ sub clickables {
     $self->xpath('//*[@onclick]', %options);
 };
 
-=head2 C<< $mech->xpath $query, %options >>
-
-    my $link = $mech->xpath('//a[id="clickme"]', one => 1);
-    # croaks if there is no link or more than one link found
-
-    my @para = $mech->xpath('//p');
-    # Collects all paragraphs
-
-Runs an XPath query in Firefox against the current document.
-
-The options allow the following keys:
-
-=over 4
-
-=item *
-
-C<< document >> - document in which the query is to be executed. Use this to
-search a node within a specific subframe of C<< $mech->document >>.
-
-=item *
-
-C<< frames >> - if true, search all documents in all frames and iframes.
-This may or may not conflict with C<node>. This will default to the
-C<frames> setting of the WWW::Mechanize::Firefox object.
-
-=item *
-
-C<< node >> - node relative to which the query is to be executed
-
-=item *
-
-C<< single >> - If true, ensure that only one element is found. Otherwise croak
-or carp, depending on the C<autodie> parameter.
-
-=item *
-
-C<< one >> - If true, ensure that at least one element is found. Otherwise croak
-or carp, depending on the C<autodie> parameter.
-
-=item *
-
-C<< maybe >> - If true, ensure that at most one element is found. Otherwise
-croak or carp, depending on the C<autodie> parameter.
-
-=item *
-
-C<< all >> - If true, return all elements found. This is the default.
-You can use this option if you want to use C<< ->xpath >> in scalar context
-to count the number of matched elements, as it will otherwise emit a warning
-for each usage in scalar context without any of the above restricting options.
-
-=back
-
-Returns the matched nodes.
-
-You can pass in a list of queries as an array reference for the first parameter.
-
-This is a method that is not implemented in WWW::Mechanize.
-
-In the long run, this should go into a general plugin for
-L<WWW::Mechanize>.
-
-=cut
-
-sub xpath {
-    my ($self,$query,%options) = @_;
-    if ('ARRAY' ne (ref $query||'')) {
-        $query = [$query];
-    };
-    
-    if ($options{ node }) {
-        $options{ document } ||= $options{ node }->{ownerDocument};
-        #warn "Have node, searching below node";
-    } else {
-        $options{ document } ||= $self->document;
-        #$options{ node } = $options{ document };
-    };
-    
-    $options{ user_info } ||= join " or ", map {qq{'$_'}} @$query;
-    my $single = delete $options{ single };
-    my $first  = delete $options{ one };
-    my $maybe  = delete $options{ maybe };
-    
-    # Construct some helper variables
-    my $zero_allowed = not ($single or $first);
-    my $two_allowed  = not( $single or $maybe );
-    my $return_first = ($single or $first or $maybe);
-    #warn "Zero: $zero_allowed";
-    #warn "Two : $two_allowed";
-    #warn "Ret : $return_first";
-    
-    # Sanity check for the common error of
-    # my $item = $mech->xpath("//foo");
-    if (! exists $options{ all } and not ($return_first)) {
-        $self->signal_condition(join "\n",
-            "You asked for many elements but seem to only want a single item.",
-            "Did you forget to pass the 'single' option with a true value?",
-            "Pass 'all => 1' to suppress this message and receive the count of items.",
-        ) if defined wantarray and !wantarray;
-    };
-    
-    if (not exists $options{ frames }) {
-        $options{frames} = $self->{frames};
-    };
-    
-    my @res;
-    
-    DOCUMENTS: {            
-        my @documents = $options{ document };
-        #warn "Invalid root document" unless $options{ document };
-        
-        # recursively join the results of sub(i)frames if wanted
-        # This should maybe go into the loop to expand every frame as we descend
-        # into the available subframes
-
-        while (@documents) {
-            my $doc = shift @documents;
-            #warn "Invalid document" unless $doc;
-
-            my $n = $options{ node } || $doc;
-            #warn "$nesting>Searching @$query in $doc->{title}";
-            push @res, map { $doc->__xpath($_, $n) } @$query;
-            
-            # A small optimization to return if we already have enough elements
-            # We can't do this on $return_first as there might be more elements
-            last DOCUMENTS if @res and $first;        
-            
-            if ($options{ frames } and not $options{ node }) {
-                #warn "$nesting>Expanding below " . $doc->{title};
-                #local $nesting .= "--";
-                my @d = $self->expand_frames( $options{ frames }, $doc );
-                #warn "Found $_->{title}" for @d;
-                push @documents, @d;
-            };
-        };
-    };
-    
-    if (! $zero_allowed and @res == 0) {
-        $self->signal_condition( "No elements found for $options{ user_info }" );
-    };
-    
-    if (! $two_allowed and @res > 1) {
-        $self->highlight_node(@res);
-        $self->signal_condition( (scalar @res) . " elements found for $options{ user_info }" );
-    };
-    
-    return $return_first ? $res[0] : @res;
-};
-
-=head2 C<< $mech->selector $css_selector, %options >>
-
-  my @text = $mech->selector('p.content');
-
-Returns all nodes matching the given CSS selector. If
-$css_selector is an array reference, it returns
-all nodes matched by any of the CSS selectors in the array.
-
-This takes the same options that C<< ->xpath >> does.
-
-In the long run, this should go into a general plugin for
-L<WWW::Mechanize>.
-
-=cut
-
-sub selector {
-    my ($self,$query,%options) = @_;
-    $options{ user_info } ||= "CSS selector '$query'";
-    if ('ARRAY' ne (ref $query || '')) {
-        $query = [$query];
-    };
-    my @q = map { selector_to_xpath($_); } @$query;
-    $self->xpath(\@q, %options);
-};
-
 =head2 C<< $mech->expand_frames $spec >>
 
   my @frames = $mech->expand_frames();
